/**
 * Copies the Location data (animation keyframes) from the first mask
 * of the selected video event's Bzier Masking FX to a selected
 * video event's Picture-In-Picture (PiP) FX
 *
 * To use this script:
 *
 * 1) Create track motion on an event using Bzier mask FX
 * 2) Apply a PiP FX to the event (from a different track) that you want to follow along with the motion on the original event
 * 3) Adjust location, scale and angle of PiP FX for a certain point in time.
 * 4) Select both events (hold ctrl key)
 * 5) Run the script
 *
 * Revision Date: Sep 5 2018
 * Copyright: MAGIX AG 
 **/
using System.Collections.Generic;
using System.Windows.Forms;
using ScriptPortal.Vegas;

public class EntryPoint
{
    Vegas myVegas;
    const string missingSelectionString = "Please select the video event to which you've applied tracking and the overlapping video event on a different track that holds the media you want to pin to the tracking.";
    const string missingTrackingDataString = "You must first apply the B\u00E9zier masking plug-in to one of the selected events and add motion tracking data for Mask 1.";
    const string missingPipFxString = "You must first add the Picture in Picture plug-in to the event that you want to pin to the motion tracked event.";

    public void FromVegas(Vegas vegas)
    {
        myVegas = vegas;

        VideoEvent bezierTrack = null;
        OFXDouble2DParameter locationTracking = null;

        List<TrackEvent> trackEvents = FindAllSelectedEventsUnderCursor();

        if (trackEvents.Count < 2)
        {
            MessageBox.Show(missingSelectionString, "No selected event", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            return;
        }
        else
        {
            bool hasBezierFx = false;
            foreach (VideoEvent trackEvent in trackEvents)
            {
                List<Effect> fxList = new List<Effect>(trackEvent.Effects);
                Effect bezierFX = fxList.Find(x => (x.PlugIn.UniqueID == "{Svfx:com.sonycreativesoftware:bzmasking}"));
                hasBezierFx = bezierFX != null;

                if (hasBezierFx)
                {
                    List<OFXParameter> ofxList = new List<OFXParameter>(bezierFX.OFXEffect.Parameters);
                    OFXParameter param = ofxList.Find(x => (x.Name == "Location_1"));

                    if (param != null)
                    {
                        locationTracking = (OFXDouble2DParameter)param;
                        bezierTrack = trackEvent;
                        trackEvents.Remove(trackEvent);
                        break;
                    }
                }
            }
            if (!hasBezierFx)
            {
                MessageBox.Show(missingTrackingDataString, "VEGAS B\u00E9zier Masking not applied", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }
        }

        if (locationTracking == null || locationTracking.Keyframes.Count == 0)
        {
            MessageBox.Show(missingTrackingDataString, "No tracking data", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            return;
        }
        else
        {
            bool hasPipFx = false;
            foreach (VideoEvent trackEvent in trackEvents)
            {
                List<Effect> fxList = new List<Effect>(trackEvent.Effects);
                Effect pipFX = fxList.Find(x => (x.PlugIn.UniqueID == "{Svfx:com.sonycreativesoftware:pictureinpicture}"));
                hasPipFx = pipFX != null;

                if (hasPipFx)
                {
                    Timecode startDiff = trackEvent.Start - bezierTrack.Start;

                    List<OFXParameter> ofxList = new List<OFXParameter>(pipFX.OFXEffect.Parameters);
                    OFXParameter param = ofxList.Find(x => (x.Name == "Location"));

                    if (param != null)
                    {
                        OFXDouble2DParameter pipLocation = (OFXDouble2DParameter)param;

                        OFXDouble2D initLocationAtCursorPosition = pipLocation.GetValueAtTime(myVegas.Transport.CursorPosition - bezierTrack.Start);
                        OFXDouble2D locationAtCursorPosition = locationTracking.GetValueAtTime(myVegas.Transport.CursorPosition - bezierTrack.Start);
                        OFXDouble2D diff = new OFXDouble2D();
                        diff.X = locationAtCursorPosition.X - initLocationAtCursorPosition.X;
                        diff.Y = locationAtCursorPosition.Y - initLocationAtCursorPosition.Y;

                        pipLocation.Keyframes.Clear();

                        foreach (var keyframe in locationTracking.Keyframes)
                        {
                            OFXDouble2D tmpValue = keyframe.Value;
                            tmpValue.X -= diff.X;
                            tmpValue.Y -= diff.Y;
                            pipLocation.SetValueAtTime(keyframe.Time - startDiff, tmpValue);
                        }
                    }
                }
                else
                {
                    MessageBox.Show(missingPipFxString, "No PiP FX", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    return;
                }
            }
        }
    }

    /// <summary>
    /// Returns the first selected event that's under the cursor
    /// </summary>
    /// <returns>The first selected event or null if no event selected</returns>
    List<TrackEvent> FindAllSelectedEventsUnderCursor()
    {
        List<TrackEvent> selectedEventsAcrossTracks = new List<TrackEvent>();

        foreach (Track track in myVegas.Project.Tracks)
        {
            foreach (TrackEvent trackEvent in track.Events)
            {
                if (trackEvent.Selected && trackEvent.Start <= myVegas.Transport.CursorPosition && trackEvent.End >= myVegas.Transport.CursorPosition)
                {
                    selectedEventsAcrossTracks.Add(trackEvent);
                }
            }
        }

        return selectedEventsAcrossTracks;
    }
}
